@@ -4,14 +4,14 @@
 ;; Copyright (C) 1997-1999 Richard M. Heiberger <rmh@fisher.stat.temple.edu>
 ;;			Kurt Hornik <hornik@ci.tuwien.ac.at>
 ;;			Martin Maechler <maechler@stat.math.ethz.ch>
-;;			A.J. (Tony) Rossini <rossini@stat.sc.edu>
+;;			A.J. (Tony) Rossini <rossini@u.washington.edu>
 
 ;; Author: David Smith <dsmith@stats.adelaide.edu.au>
-;; Maintainer: A.J. Rossini <rossinI@biostat.washington.edu>
+;; Maintainer: A.J. Rossini <rossini@u.washington.edu>
 ;; Created: 7 Jan 1994
-;; Modified: $Date: 1999/07/22 13:02:04 $
-;; Version: $Revision: 5.8 $
-;; RCS: $Id: ess-mode.el,v 5.8 1999/07/22 13:02:04 maechler Exp $
+;; Modified: $Date: 2002/01/15 01:52:54 $
+;; Version: $Revision: 5.20 $
+;; RCS: $Id: ess-mode.el,v 5.20 2002/01/15 01:52:54 rmh Exp $
 
 ;; This file is part of ESS
 
@@ -40,32 +40,50 @@
 (require 'ess)
 
 ;;; AJR: THIS IS GROSS AND DISGUSTING (but I wrote it).
-(autoload 'ess-mode-minibuffer-map "ess-inf" "" nil 'keymap)
-(autoload 'ess-read-object-name "ess-inf" "" nil)
-(autoload 'ess-list-object-completions "ess-inf" "" nil)
+;;; MM:  and I had to add all other 'ess-eval-*** ...
+;;; >>> why not just do the obvious instead of all these ? Namely,
+;;; (require 'ess-inf)
+;;; ------------------ ?
+;;; AJR: The reason is that we ONLY need to load ess-inf for the
+;;; functions which are interactive in nature.   We don't want to load
+;;; it when we are only editing.
 
-(autoload 'ess-load-file "ess-inf" "" nil)
-(autoload 'get-ess-process "ess-inf" "" nil)
-(autoload 'ess-switch-to-ESS "ess-inf" "" nil)
-(autoload 'ess-request-a-process "ess-inf" "" nil)
-(autoload 'ess-modtime-gt "ess-inf" "" nil)
-(autoload 'ess-create-temp-buffer "ess-inf" "" nil)
-(autoload 'ess-command "ess-inf" "" nil)
-(autoload 'ess-eval-visibly "ess-inf" "" nil)
-(autoload 'ess-make-buffer-current "ess-inf" "" nil)
-(autoload 'ess-object-modtime "ess-inf" "" nil)
-(autoload 'ess-display-temp-buffer "ess-inf" "" nil)
-(autoload 'ess-force-buffer-current "ess-inf" "" nil)
-(autoload 'ess-switch-process "ess-inf" "" nil)
-(autoload 'ess-quit           "ess-inf" "" nil)
+(autoload 'ess-mode-minibuffer-map	"ess-inf" "" nil 'keymap)
+(autoload 'ess-read-object-name		"ess-inf" "" nil)
+(autoload 'ess-list-object-completions	"ess-inf" "" nil)
+
+(autoload 'ess-eval-linewise		"ess-inf" "" nil)
+(autoload 'ess-eval-paragraph		"ess-inf" "" nil)
+(autoload 'ess-eval-region		"ess-inf" "" nil)
+(autoload 'ess-eval-buffer		"ess-inf" "" nil)
+(autoload 'ess-eval-function		"ess-inf" "" nil)
+(autoload 'ess-eval-line		"ess-inf" "" nil)
+(autoload 'ess-eval-line-and-step	"ess-inf" "" nil)
+(autoload 'ess-eval-region-and-go	"ess-inf" "" nil)
+(autoload 'ess-eval-buffer-and-go	"ess-inf" "" nil)
+(autoload 'ess-eval-function-and-go	"ess-inf" "" nil)
+(autoload 'ess-eval-line-and-go		"ess-inf" "" nil)
+
+(autoload 'ess-load-file		"ess-inf" "" nil)
+(autoload 'ess-switch-process		"ess-inf" "" nil)
+(autoload 'ess-switch-to-ESS		"ess-inf" "" nil)
+(autoload 'ess-request-a-process	"ess-inf" "" nil)
+(autoload 'get-ess-process		"ess-inf" "" nil)
+(autoload 'ess-command			"ess-inf" "" nil)
+(autoload 'ess-create-temp-buffer	"ess-inf" "" nil)
+(autoload 'ess-display-temp-buffer	"ess-inf" "" nil)
+(autoload 'ess-force-buffer-current	"ess-inf" "" nil)
+(autoload 'ess-make-buffer-current	"ess-inf" "" nil)
+(autoload 'ess-modtime-gt		"ess-inf" "" nil)
+(autoload 'ess-object-modtime		"ess-inf" "" nil)
+(autoload 'ess-quit			"ess-inf" "" nil)
+
+
+(defun ess-line-end-position (&optional N)
+  (save-excursion
+    (end-of-line N)
+    (point)))
 
-;; (line-end-position N) exists in Emacs 20.3, but not (eg in 19.34)
-(if (not (fboundp 'line-end-position))
-    (defun line-end-position (&optional N)
-      (save-excursion
-	(end-of-line N)
-	(point)))
-)
 
  ; ESS mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -81,12 +99,11 @@
 
 (if ess-eval-map
     nil
-  (cond ((string-match "XEmacs\\|Lucid" emacs-version)
-	 ;; Code for XEmacs
- 	 (setq ess-eval-map (make-keymap)))
-	((not (string-match "XEmacs\\|Lucid" emacs-version))
-	 ;; Code for GNU Emacs
-	 (setq ess-eval-map (make-sparse-keymap))))
+  (if ess-running-xemacs
+      ;; Code for XEmacs
+      (setq ess-eval-map (make-keymap))
+    ;; else code for GNU Emacs
+    (setq ess-eval-map (make-sparse-keymap)))
 
   (define-key ess-eval-map "\C-r"    'ess-eval-region)
   (define-key ess-eval-map "\M-r"    'ess-eval-region-and-go)
@@ -95,7 +112,7 @@
   (define-key ess-eval-map "\C-f"    'ess-eval-function)
   (define-key ess-eval-map "\M-f"    'ess-eval-function-and-go)
   (define-key ess-eval-map "\C-x"    'ess-eval-function)
-  (define-key ess-eval-map "\C-n"    'ess-eval-line-and-next-line)
+  (define-key ess-eval-map "\C-n"    'ess-eval-line-and-step)
   (define-key ess-eval-map "\C-j"    'ess-eval-line)
   (define-key ess-eval-map "\M-j"    'ess-eval-line-and-go))
 
@@ -103,117 +120,119 @@
 (if ess-mode-map
     nil
 
-  (cond ((string-match "XEmacs\\|Lucid" emacs-version)
-	 ;; Code for XEmacs
- 	 (setq ess-mode-map (make-keymap))
-	 (set-keymap-parent ess-mode-map text-mode-map)) ;; was comint?!?
-	((not (string-match "XEmacs\\|Lucid" emacs-version))
-	 ;; Code for GNU Emacs
-	 (setq ess-mode-map (make-sparse-keymap))))
+  (if ess-running-xemacs
+      (progn ;; Code for XEmacs
+	(setq ess-mode-map (make-keymap))
+	(set-keymap-parent ess-mode-map text-mode-map)) ;; was comint?!?
+    ;; else code for GNU Emacs
+    (setq ess-mode-map (make-sparse-keymap)))
 
   ;; By popular demand:
-  (define-key ess-mode-map "\C-m"        'newline-and-indent)
+  (define-key ess-mode-map "\C-m"	'newline-and-indent); = [RETURN]
 
-  (define-key ess-mode-map "\C-c\C-r"    'ess-eval-region)
-  (define-key ess-mode-map "\C-c\M-r"    'ess-eval-region-and-go)
-  (define-key ess-mode-map "\C-c\C-b"    'ess-eval-buffer)
-  (define-key ess-mode-map "\C-c\M-b"    'ess-eval-buffer-and-go)
-  (define-key ess-mode-map "\C-c\C-f"    'ess-eval-function)
-  (define-key ess-mode-map "\C-c\M-f"    'ess-eval-function-and-go)
-  (define-key ess-mode-map "\M-\C-x"     'ess-eval-function)
-  (define-key ess-mode-map "\C-c\C-n"    'ess-eval-line-and-next-line)
-  (define-key ess-mode-map "\C-c\C-j"    'ess-eval-line)
-  (define-key ess-mode-map "\C-c\M-j"    'ess-eval-line-and-go)
-  (define-key ess-mode-map "\M-\C-a"     'ess-beginning-of-function)
-  (define-key ess-mode-map "\M-\C-e"     'ess-end-of-function)
-  (define-key ess-mode-map "\C-c\C-y"    'ess-switch-to-ESS)
-  (define-key ess-mode-map "\C-c\C-z"    'ess-switch-to-end-of-ESS)
-  (define-key ess-mode-map "\C-c\C-l"    'ess-load-file)
-  (define-key ess-mode-map "\C-c\C-v"    'ess-display-help-on-object)
-  (define-key ess-mode-map "\C-c\C-d"    'ess-dump-object-into-edit-buffer)
+  (define-key ess-mode-map "\C-c\C-r"	'ess-eval-region)
+  (define-key ess-mode-map "\C-c\M-r"	'ess-eval-region-and-go)
+  (define-key ess-mode-map "\C-c\C-b"	'ess-eval-buffer)
+  (define-key ess-mode-map "\C-c\M-b"	'ess-eval-buffer-and-go)
+  (define-key ess-mode-map "\C-c\C-f"	'ess-eval-function)
+  (define-key ess-mode-map "\C-c\M-f"	'ess-eval-function-and-go)
+  (define-key ess-mode-map "\M-\C-x"	'ess-eval-function)
+  (define-key ess-mode-map "\C-c\C-n"	'ess-eval-line-and-step)
+  (define-key ess-mode-map "\C-c\C-j"	'ess-eval-line)
+  (define-key ess-mode-map "\C-c\M-j"	'ess-eval-line-and-go)
+  (define-key ess-mode-map "\M-\C-a"	'ess-beginning-of-function)
+  (define-key ess-mode-map "\M-\C-e"	'ess-end-of-function)
+  (define-key ess-mode-map "\C-xnd"	'ess-narrow-to-defun)
+  (define-key ess-mode-map "\C-c\C-y"	'ess-switch-to-ESS)
+  (define-key ess-mode-map "\C-c\C-z"	'ess-switch-to-end-of-ESS)
+  (define-key ess-mode-map "\C-c\C-l"	'ess-load-file)
+  (define-key ess-mode-map "\C-c\C-v"	'ess-display-help-on-object)
+  (define-key ess-mode-map "\C-c\C-d"	'ess-dump-object-into-edit-buffer)
 ;(define-key ess-mode-map "\C-c5\C-d"'ess-dump-object-into-edit-buffer-other-frame)
-  (define-key ess-mode-map "\C-c\C-s"    'ess-switch-process) ; use a
+  (define-key ess-mode-map "\C-c\C-s"	'ess-switch-process) ; use a
 					; different process for the buffer.
-  (define-key ess-mode-map "\C-c\C-t"    'ess-execute-in-tb)
-  (define-key ess-mode-map "\C-c\t"      'ess-complete-object-name)
-  (define-key ess-mode-map "\M-\t"       'comint-replace-by-expanded-filename)
-  (define-key ess-mode-map "\M-?"        'ess-list-object-completions)
+  (define-key ess-mode-map "\C-c\C-t"	'ess-execute-in-tb)
+  (define-key ess-mode-map "\C-c\t"	'ess-complete-object-name)
+  (define-key ess-mode-map "\M-\t"	'comint-replace-by-expanded-filename)
+  (define-key ess-mode-map "\M-?"	'ess-list-object-completions)
   ;; wrong here (define-key ess-mode-map "\C-c\C-k" 'ess-request-a-process)
-  (define-key ess-mode-map "\C-c\C-k"    'ess-force-buffer-current)
-  (define-key ess-mode-map "\C-c`"       'ess-parse-errors) ; \C-x reserved!
-  (define-key ess-mode-map "{"           'ess-electric-brace)
-  (define-key ess-mode-map "}"           'ess-electric-brace)
-  (define-key ess-mode-map "\e\C-h"      'ess-mark-function)
-  (define-key ess-mode-map "\e\C-q"      'ess-indent-exp)
-  (define-key ess-mode-map "\177"        'backward-delete-char-untabify)
-  (define-key ess-mode-map "\t"          'ess-indent-command)
-  (define-key ess-mode-map "\C-c\C-q"    'ess-quit)
-  (define-key ess-mode-map "\C-c\C-e"    ess-eval-map))
-
+  (define-key ess-mode-map "\C-c\C-k"	'ess-force-buffer-current)
+  (define-key ess-mode-map "\C-c`"	'ess-parse-errors) ; \C-x reserved!
+  (define-key ess-mode-map "{"		'ess-electric-brace)
+  (define-key ess-mode-map "}"		'ess-electric-brace)
+  (define-key ess-mode-map "\e\C-h"	'ess-mark-function)
+  (define-key ess-mode-map "\e\C-q"	'ess-indent-exp)
+  (define-key ess-mode-map "\177"	'backward-delete-char-untabify)
+  (define-key ess-mode-map "\t"		'ess-indent-command)
+  (define-key ess-mode-map "\C-c\C-q"	'ess-quit)
+  (define-key ess-mode-map "\C-c\C-e"	ess-eval-map))
 
+(require 'noweb-mode)
 (easy-menu-define
  ess-mode-menu ess-mode-map
- "Menu for use in ess-mode"
+ "Menu for use in `ess-mode'."
  '("ESS" ; ESS-mode
-   ["Load file"  ess-load-file t]
+   ["What is this? (beta)"    ess-mouse-me                      t]
+   ["Load file"	 ess-load-file t]
    ("Eval and Go"
-    ["Eval buffer"   ess-eval-buffer-and-go            t]
-    ["Eval region"   ess-eval-region-and-go            t]
-    ["Eval function" ess-eval-function-and-go          t]
-    ["Eval line"     ess-eval-line-and-go              t]
-    ["Eval chunk"    ess-eval-chunk-and-go    noweb-mode]
-    ["Eval thread"   ess-eval-thread-and-go   noweb-mode]
-    ["About"         (ess-goto-info "Evaluating code") t]
+    ["Eval buffer"	ess-eval-buffer-and-go		  t]
+    ["Eval region"	ess-eval-region-and-go		  t]
+    ["Eval function"	ess-eval-function-and-go	  t]
+    ["Eval line"	ess-eval-line-and-go		  t]
+    ["Eval chunk"	ess-eval-chunk-and-go	 noweb-mode]
+    ["Eval thread"	ess-eval-thread-and-go	 noweb-mode]
+    ["About"		(ess-goto-info "Evaluating code") t]
     )
    ("ESS Eval"
-    ["Eval buffer"       ess-eval-buffer                   t]
-    ["Eval region"       ess-eval-region                   t]
-    ["Eval function"     ess-eval-function                 t]
-    ["Step through line" ess-eval-line-and-next-line       t]
-    ["Enter expression"  ess-execute-in-tb                 t]
-    ["Eval line"         ess-eval-line                     t]
-    ["Eval chunk"        ess-eval-chunk           noweb-mode]
-    ["Eval thread"       ess-eval-thread          noweb-mode]
-    ["About"             (ess-goto-info "Evaluating code") t]
+    ["Eval buffer"	ess-eval-buffer			  t]
+    ["Eval region"	ess-eval-region			  t]
+    ["Eval function"	ess-eval-function		  t]
+    ["Enter expression" ess-execute-in-tb		  t]
+    ["Eval line"	ess-eval-line			  t]
+    ["Eval line & step" ess-eval-line-and-step		  t]
+    ["Eval chunk"	ess-eval-chunk		 noweb-mode]
+    ["Eval thread"	ess-eval-thread		 noweb-mode]
+    ["About"		(ess-goto-info "Evaluating code") t]
     )
    ("Motion..."
-    ["Edit new object"       ess-dump-object-into-edit-buffer t]
-    ["Goto end of ESS buffer"  ess-switch-to-end-of-ESS       t]
-    ["Switch to ESS buffer"    ess-switch-to-ESS              t]
-    ["End of function"	    ess-end-of-function               t]
-    ["Beginning of function" ess-beginning-of-function        t])
+    ["Edit new object"		ess-dump-object-into-edit-buffer t]
+    ["Goto end of ESS buffer"	ess-switch-to-end-of-ESS	t]
+    ["Switch to ESS buffer"	ess-switch-to-ESS		t]
+    ["End of function"		ess-end-of-function		t]
+    ["Beginning of function"	ess-beginning-of-function	t])
    ("ESS list..."
-    ["Backward list"         backward-list                   t]
-    ["Forward list"          forward-list                    t]
-    ["Next parenthesis"      down-list                       t]
-    ["Enclosing parenthesis" backward-up-list                t]
-    ["Backward sexp"         backward-sexp                   t]
-    ["Forward sexp"          forward-sexp                    t]
-    ["About"                 (Info-goto-node "(Emacs)Lists") t]
+    ["Backward list"		backward-list			t]
+    ["Forward list"		forward-list			t]
+    ["Next parenthesis"		down-list			t]
+    ["Enclosing parenthesis"	backward-up-list		t]
+    ["Backward sexp"		backward-sexp			t]
+    ["Forward sexp"		forward-sexp			t]
+    ["About"			(Info-goto-node "(Emacs)Lists") t]
     )
    ("ESS Edit"
-    ["Complete Filename" comint-replace-by-expanded-filename t]
-    ["Complete Object"   ess-complete-object-name            t]
-    ["Kill sexp"         kill-sexp                           t]
-    ["Mark function"     ess-mark-function                   t]
-    ["Indent expression" ess-indent-exp                      t]
-    ["Indent line"       ess-indent-command                  t]
-    ["Undo"              undo                                t]
-    ["About"             (ess-goto-info "Edit buffer")       t]
+    ["Complete Filename" comint-replace-by-expanded-filename	t]
+    ["Complete Object"	 ess-complete-object-name		t]
+    ["Kill sexp"	 kill-sexp				t]
+    ["Mark function"	 ess-mark-function			t]
+    ["Indent expression" ess-indent-exp				t]
+    ["Indent line"	 ess-indent-command			t]
+    ["Undo"		 undo					t]
+    ["About"		 (ess-goto-info "Edit buffer")		t]
     )
+   ["Switch Process"	ess-switch-process		t]
    "------"
-   ["Describe"         describe-mode                t]
-   ["About"            (ess-goto-info "Editing")    t]
-   ["Send bug report"  ess-submit-bug-report        t]
+   ["Describe"		describe-mode			t]
+   ["About"		(ess-goto-info "Editing")	t]
+   ["Send bug report"	ess-submit-bug-report		t]
    ))
 
 (defun ess-mode-xemacs-menu ()
-  "Hook to install ess-mode menu for XEmacs (w/ easymenu)"
+  "Hook to install `ess-mode' menu for XEmacs (w/ easymenu)."
   (if 'ess-mode
         (easy-menu-add ess-mode-menu)
     (easy-menu-remove ess-mode-menu)))
 
-(if (string-match "XEmacs" emacs-version)
+(if ess-running-xemacs
     (add-hook 'ess-mode-hook 'ess-mode-xemacs-menu))
 
 (defun ess-mode (&optional alist proc-name)
@@ -229,21 +248,21 @@ Entry to this mode runs the hooks in ess-mode-hook.
 
 You can send text to the inferior ESS process from other buffers containing
 ESS source.
-    ess-eval-region sends the current region to the ESS process.
-    ess-eval-buffer sends the current buffer to the ESS process.
-    ess-eval-function sends the current function to the ESS process.
-    ess-eval-line sends the current line to the ESS process.
-    ess-beginning-of-function and ess-end-of-function move the point to
+    `ess-eval-region' sends the current region to the ESS process.
+    `ess-eval-buffer' sends the current buffer to the ESS process.
+    `ess-eval-function' sends the current function to the ESS process.
+    `ess-eval-line' sends the current line to the ESS process.
+    `ess-beginning-of-function' and `ess-end-of-function' move the point to
         the beginning and end of the current ESS function.
-    ess-switch-to-ESS switches the current buffer to the ESS process buffer.
-    ess-switch-to-end-of-ESS switches the current buffer to the ESS process
+    `ess-switch-to-ESS' switches the current buffer to the ESS process buffer.
+    `ess-switch-to-end-of-ESS' switches the current buffer to the ESS process
         buffer and puts point at the end of it.
 
-    ess-eval-region-and-go, ess-eval-buffer-and-go,
-        ess-eval-function-and-go, and ess-eval-line-and-go switch to the S
+    `ess-eval-region-and-go', `ess-eval-buffer-and-go',
+        `ess-eval-function-and-go', and `ess-eval-line-and-go' switch to the S
         process buffer after sending their text.
 
-    ess-load-file sources a file of commands to the ESS process.
+    `ess-load-file' sources a file of commands to the ESS process.
 
 \\[ess-indent-command] indents for ESS code.
 \\[backward-delete-char-untabify] converts tabs to spaces as it moves back.
@@ -258,42 +277,42 @@ Comments are indented in a similar way to Emacs-lisp mode:
 \\[ess-indent-exp] command indents each line of the ESS grouping following point.
 
 Variables controlling indentation style:
- ess-tab-always-indent
+ `ess-tab-always-indent'
     Non-nil means TAB in ESS mode should always reindent the current line,
     regardless of where in the line point is when the TAB command is used.
- ess-auto-newline
+ `ess-auto-newline'
     Non-nil means automatically newline before and after braces inserted in S
     code.
- ess-indent-level
+ `ess-indent-level'
     Indentation of ESS statements within surrounding block.
     The surrounding block's indentation is the indentation of the line on
     which the open-brace appears.
- ess-continued-statement-offset
+ `ess-continued-statement-offset'
     Extra indentation given to a substatement, such as the then-clause of an
     if or body of a while.
- ess-continued-brace-offset
+ `ess-continued-brace-offset'
     Extra indentation given to a brace that starts a substatement.
     This is in addition to ess-continued-statement-offset.
- ess-brace-offset
+ `ess-brace-offset'
     Extra indentation for line if it starts with an open brace.
- ess-arg-function-offset
+ `ess-arg-function-offset'
     Extra indent for internal substatements of function `foo' that called
     in `arg=foo(...)' form.
    If not number, the statements are indented at open-parenthesis following
    `foo'.
- ess-expression-offset
+ `ess-expression-offset'
     Extra indent for internal substatements of `expression' that specified
     in `obj <- expression(...)' form.
     If not number, the statements are indented at open-parenthesis following
     `expression'.
- ess-brace-imaginary-offset
+ `ess-brace-imaginary-offset'
     An open brace following other text is treated as if it were
     this far to the right of the start of its line.
- ess-else-offset
+ `ess-else-offset'
     Extra indentation for line if it starts with `else'.
- ess-close-brace-offset
+ `ess-close-brace-offset'
     Extra indentation for closing braces.
- ess-fancy-comments
+ `ess-fancy-comments'
     Non-nil means distinguish between #, ##, and ### for indentation.
 
 Furthermore, \\[ess-set-style] command enables you to set up predefined ess-mode
@@ -307,9 +326,9 @@ indentation style. At present, predefined style are `BSD', `GNU', `K&R', `C++',
 	   ess-language
 	   ess-dialect
 	   (current-buffer)))
-;;  (ess-write-to-dribble-buffer
-;;   (format "(ess-mode-1.2): ess-process= %s \n"
-;;	   (ess-local-process-name ess-local-process-name "none")))
+  ;; (ess-write-to-dribble-buffer
+  ;;  (format "(ess-mode-1.2): ess-process= %s \n"
+  ;;   (ess-local-process-name ess-local-process-name "none")))
   (ess-write-to-dribble-buffer
    (format "(ess-mode-1.5): alist=%s \n"
 	   alist))
@@ -324,39 +343,37 @@ indentation style. At present, predefined style are `BSD', `GNU', `K&R', `C++',
   (use-local-map ess-mode-map)
   (set-syntax-table ess-mode-syntax-table)
 
-  ;;; Keep <tabs> out of the code.
+  ;; Keep <tabs> out of the code.
   (make-local-variable 'indent-tabs-mode)
   (setq indent-tabs-mode nil)
 
-
-;;  (make-local-variable 'paragraph-start)
-;;  (setq paragraph-start (concat "^$\\|" page-delimiter))
-;;  (make-local-variable 'paragraph-separate)
-;;  (setq paragraph-separate paragraph-start)
-;;  (make-local-variable 'paragraph-ignore-fill-prefix)
-;;  (setq paragraph-ignore-fill-prefix t)
-;;  (make-local-variable 'indent-line-function)
-;;  (setq indent-line-function 'ess-indent-line)
-;;  (make-local-variable 'require-final-newline)
-;;  (setq require-final-newline t)
-;;  (make-local-variable 'comment-start)
-;;  (setq comment-start "#")
-;;  (make-local-variable 'comment-start-skip)
-;;  (setq comment-start-skip "#+ *")
-;;  (make-local-variable 'comment-column)
-;;  (setq comment-column 40)
-;;  (make-local-variable 'comment-indent-function)
-;;  (setq comment-indent-function 'ess-comment-indent)
-;;  (make-local-variable 'parse-sexp-ignore-comments)
-;;  (setq parse-sexp-ignore-comments t)
-;;  (ess-set-style ess-default-style)
-;;  (make-local-variable 'ess-local-process-name)
-;;  (make-local-variable 'ess-keep-dump-files)
+  ;;  (make-local-variable 'paragraph-start)
+  ;;  (setq paragraph-start (concat "^$\\|" page-delimiter))
+  ;;  (make-local-variable 'paragraph-separate)
+  ;;  (setq paragraph-separate paragraph-start)
+  ;;  (make-local-variable 'paragraph-ignore-fill-prefix)
+  ;;  (setq paragraph-ignore-fill-prefix t)
+  ;;  (make-local-variable 'indent-line-function)
+  ;;  (setq indent-line-function 'ess-indent-line)
+  ;;  (make-local-variable 'require-final-newline)
+  ;;  (setq require-final-newline t)
+  ;;  (make-local-variable 'comment-start)
+  ;;  (setq comment-start "#")
+  ;;  (make-local-variable 'comment-start-skip)
+  ;;  (setq comment-start-skip "#+ *")
+  ;;  (make-local-variable 'comment-column)
+  ;;  (setq comment-column 40)
+  ;;  (make-local-variable 'comment-indent-function)
+  ;;  (setq comment-indent-function 'ess-comment-indent)
+  ;;  (make-local-variable 'parse-sexp-ignore-comments)
+  ;;  (setq parse-sexp-ignore-comments t)
+  ;;  (ess-set-style ess-default-style)
+  ;;  (make-local-variable 'ess-local-process-name)
+  ;;  (make-local-variable 'ess-keep-dump-files)
   (put 'ess-local-process-name 'permanent-local t) ; protect from RCS
   (setq mode-line-process ;; AJR: in future, XEmacs will use modeline-process.
 	'(" [" (ess-local-process-name ess-local-process-name "none") "]"))
 
-
   (ess-load-object-name-db-file)
   (run-hooks 'ess-mode-hook)
   (ess-write-to-dribble-buffer "\nFinished setting up ESS-mode.\n"))
@@ -366,7 +383,7 @@ indentation style. At present, predefined style are `BSD', `GNU', `K&R', `C++',
 ;;;*;;; Handy commands
 
 (defun ess-execute-in-tb ()
-  "Like ess-execute, but always evaluates in temp buffer."
+  "Like `ess-execute', but always evaluates in temp buffer."
   (interactive)
   (let ((ess-execute-in-process-buffer nil))
     (call-interactively 'ess-execute)))
@@ -380,7 +397,7 @@ indentation style. At present, predefined style are `BSD', `GNU', `K&R', `C++',
  	beg end done)
     ;;DBG (ess-write-to-dribble-buffer "ess-BEG-of-fun:")
     ;; in case we're sitting in a function header:
-    (if (search-forward "(" (line-end-position 2) t); at most end of next line
+    (if (search-forward "(" (ess-line-end-position 2) t); at most end of next line
 	(forward-char 1))
     (while (not done)
       (if (re-search-backward ess-function-pattern (point-min) t)
@@ -426,6 +443,19 @@ Optional argument for location of beginning.  Return '(beg end)."
     (ess-end-of-function beg)
     (exchange-point-and-mark)))
 
+;; Donated by Stephen Eglen, 2001-08-29:
+;; This command is analogous to `narrow-to-defun' (elisp)
+;; and `py-narrow-to-defun' (python)."
+(defun ess-narrow-to-defun ()
+  "Make text outside current function invisible.
+If text is already narrowed, this is removed before narrowing to the
+current function."
+  (interactive)
+   ;; if point is not in a function, ess-end-of-function catches the error.
+  (save-excursion
+    (widen)
+    (let* ((beg-end (ess-end-of-function)))
+      (narrow-to-region (nth 0 beg-end) (nth 1 beg-end)))))
 
 ;;*;; Loading files
 
@@ -463,7 +493,7 @@ if this is the case."
 
 (defun ess-check-source (fname)
   "If file FNAME has an unsaved buffer, offer to save it.
-Returns t if the buffer existed and was modified, but was not saved"
+Returns t if the buffer existed and was modified, but was not saved."
   (let ((buff (get-file-buffer fname)))
     ;; RMH: Corrections noted below are needed for C-c C-l to work
     ;; correctly when issued from *S* buffer.
@@ -557,14 +587,13 @@ With prefix argument, only shows the errors ESS reported."
 
 (defun ess-indent-command (&optional whole-exp)
   "Indent current line as ESS code, or in some cases insert a tab character.
-If ess-tab-always-indent is non-nil (the default), always indent current line.
-Otherwise, indent the current line only if point is at the left margin
-or in the line's indentation; otherwise insert a tab.
-
-A numeric argument, regardless of its value,
-means indent rigidly all the lines of the expression starting after point
-so that this line becomes properly indented.
-The relative indentation among the lines of the expression are preserved."
+If `ess-tab-always-indent' is non-nil (the default), always indent
+current line.  Otherwise, indent the current line only if point is at
+the left margin or in the line's indentation; otherwise insert a tab.
+A numeric argument, regardless of its value, means indent rigidly all
+the lines of the expression starting after point so that this line
+becomes properly indented.  The relative indentation among the lines
+of the expression are preserved."
   (interactive "P")
   (if whole-exp
       ;; If arg, always indent this line as S
@@ -709,7 +738,9 @@ The relative indentation among the lines of the expression are preserved."
 		  (indent-to this-indent)))
 	    ;; Indent any comment following the text.
 	    (or (looking-at comment-start-skip)
-		(if (re-search-forward comment-start-skip (save-excursion (end-of-line) (point)) t)
+		(if (re-search-forward comment-start-skip
+				       (save-excursion (end-of-line)
+						       (point)) t)
 		    (progn (indent-for-comment) (beginning-of-line))))))))))
 ;; (message "Indenting ESS expression...done")
 
@@ -870,7 +901,8 @@ Returns nil if line starts inside a string, t if in a comment."
 		 ;; indent it relative to line brace is on.
 		 ;; For open brace in column zero, don't let statement
 		 ;; start there too.  If ess-indent-level is zero,
-		 ;; use ess-brace-offset + ess-continued-statement-offset instead.
+		 ;; use ess-brace-offset +
+		 ;; ess-continued-statement-offset instead.
 		 ;; For open-braces not the first thing in a line,
 		 ;; add in ess-brace-imaginary-offset.
 		 (+ (if (and (bolp) (zerop ess-indent-level))
@@ -953,38 +985,32 @@ Returns nil if line starts inside a string, t if in a comment."
 ;;;*;;; Predefined indentation styles
 
 (defun ess-set-style (&optional style quiet)
-  "Set up the ess-mode style variables from the ess-style variable or if
-  STYLE argument is given, use that.  It makes the ESS indentation style
-  variables buffer local."
+  "Set up the `ess-mode' style variables from the `ess-style' variable
+or if STYLE argument is given, use that.  It makes the ESS indentation
+style variables buffer local."
 
   (interactive)
-
   (let ((ess-styles (mapcar 'car ess-style-alist)))
-
     (if (interactive-p)
 	(setq style
 	      (let ((style-string ; get style name with completion
 		     (completing-read
-		      (format "Set ESS mode indentation style to (default %s): "
-			      ess-default-style)
+		      (format
+		       "Set ESS mode indentation style (default %s): "
+		       ess-default-style)
 		      (vconcat ess-styles)
 		      (function (lambda (arg) (memq arg ess-styles)))
 		      )))
 		(if (string-equal "" style-string)
 		    ess-default-style
-		  (intern style-string))
-		)))
-
+		  (intern style-string)))))
     (setq style (or style ess-style)) ; use ess-style if style is nil
-
     (make-local-variable 'ess-style)
     (if (memq style ess-styles)
 	(setq ess-style style)
-      (error (concat "Bad ESS style: " style))
-      )
+      (error (concat "Bad ESS style: " style)))
     (if (not quiet)
 	(message "ESS-style: %s" ess-style))
-
     ; finally, set the indentation style variables making each one local
     (mapcar (function (lambda (ess-style-pair)
 			(make-local-variable (car ess-style-pair))
@@ -1001,7 +1027,7 @@ Returns nil if line starts inside a string, t if in a comment."
   "Edit an ESS object in its own buffer.
 
 Without a prefix argument, this simply finds the file pointed to by
-ess-source-directory. If this file does not exist, or if a
+`ess-source-directory'. If this file does not exist, or if a
 prefix argument is given, a dump() command is sent to the ESS process to
 generate the source buffer."
   (interactive
@@ -1113,15 +1139,10 @@ generate the source buffer."
 		  (down-list 1)
 		(error nil)))))))
 
-
-;; AJR: XEmacs, makes sense to dump into "other frame".
-
 (defun ess-dump-object-into-edit-buffer-other-frame (object)
   "Edit an ESS object in its own frame."
   (switch-to-buffer-other-frame (ess-dump-object-into-edit-buffer object)))
 
-
-
 (provide 'ess-mode)
 
  ; Local variables section
