@@ -6,9 +6,9 @@
 ;; Author: Richard M. Heiberger <rmh@fisher.stat.temple.edu>
 ;; Maintainer: A.J. Rossini <rossini@biostat.washington.edu>
 ;; Created: December 1998
-;; Modified: $Date: 1999/06/17 16:46:09 $
-;; Version: $Revision: 1.4 $
-;; RCS: $Id: essd-sp4.el,v 1.4 1999/06/17 16:46:09 maechler Exp $
+;; Modified: $Date: 2002/01/21 03:26:13 $
+;; Version: $Revision: 1.10 $
+;; RCS: $Id: essd-sp4.el,v 1.10 2002/01/21 03:26:13 rmh Exp $
 ;;
 ;; Keywords: start up, configuration.
 
@@ -37,19 +37,19 @@
 (require 'essl-s)
 (require 'ess-iw32)
 
-(autoload 'inferior-ess "ess-inf" "Run an ESS process")
-(autoload 'ess-mode     "ess-mode" "Edit an ESS process")
+(autoload 'inferior-ess "ess-inf" "Run an ESS process.")
+(autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
 
-; Code:
+;;; Code:
 
 (defvar S+4-dialect-name "S+4"
   "Name of 'dialect' for S-PLUS 4.x.");easily changeable in a user's .emacs
 
 (defvar inferior-S+4-multipleinstances "/MULTIPLEINSTANCES"
   "Default \"/MULTIPLEINSTANCES\" opens up a new instance of S+4 in a
-GUI window and connects it to the *S+4 ddeclient* window.  The
+GUI window and connects it to the '(ddeESS [S+4])' window.  The
 alternative nil uses an existing S+4 GUI (if there is one) and
-connects it to the *S+4 ddeclient* window.")
+connects it to the '(ddeESS [S+4])' window.")
 
 (defvar S+4-customize-alist
   '((ess-local-customize-alist     . 'S+4-customize-alist)
@@ -78,6 +78,7 @@ connects it to the *S+4 ddeclient* window.")
     (inferior-ess-exit-command     . "q()\n")
     (inferior-ess-primary-prompt   . "[a-zA-Z0-9() ]*> ?")
     (inferior-ess-secondary-prompt . "+ ?")
+    (comint-use-prompt-regexp-instead-of-fields . t) ;; emacs 21 and up
     (inferior-ess-start-file       . nil) ;"~/.ess-S+4")
     (inferior-ess-start-args       . (concat
 				      inferior-S+4-multipleinstances
@@ -85,7 +86,19 @@ connects it to the *S+4 ddeclient* window.")
 				      inferior-S+4-print-command
 				      " S_PROJ="
 				      (directory-file-name default-directory))
-				   ))
+				   )
+    (ess-STERM  . "ddeESS")
+    (ess-editor . S-editor)
+    (ess-pager  . S-pager)
+    (inferior-ess-language-start .
+				 (concat "options("
+					 "STERM='"  ess-STERM  "'"
+					 (if ess-editor 
+					     (concat ", editor='" ess-editor "'"))
+					 (if ess-pager 
+					     (concat ", pager='"  ess-pager  "'"))
+					 ")"))
+)
  "Variables to customize for S+4")
 
 (defvar Sqpe+4-customize-alist
@@ -112,9 +125,22 @@ connects it to the *S+4 ddeclient* window.")
     (inferior-ess-exit-command     . "q()\n")
     (inferior-ess-primary-prompt   . "[a-zA-Z0-9() ]*> ?")
     (inferior-ess-secondary-prompt . "+ ?")
+    (comint-use-prompt-regexp-instead-of-fields . t) ;; emacs 21 and up
     (inferior-ess-start-file       . nil) ;"~/.ess-S+4")
-    (inferior-ess-start-args       . ""))
- "Variables to customize for Sqpe+4")
+    (inferior-ess-start-args       . "")
+    (ess-STERM  . "iESS")
+    (ess-editor . S-editor)
+    (ess-pager  . S-pager)
+    (inferior-ess-language-start .
+				 (concat "options("
+					 "STERM='"  ess-STERM  "'"
+					 (if ess-editor 
+					     (concat ", editor='" ess-editor "'"))
+					 (if ess-pager 
+					     (concat ", pager='"  ess-pager  "'"))
+					 ")"))
+)
+ "Variables to customize for Sqpe+4.")
 
 
 
@@ -124,7 +150,7 @@ connects it to the *S+4 ddeclient* window.")
 ;;;     buffer.  It works as as a GUI window and we must send commands
 ;;;     to it through ddeclient.  Nonetheless, we need to give it a
 ;;;     process name and be sure that that there is a valid running
-;;;     process in the *S+4 ddeclient* buffer.  Therefore we create an
+;;;     process in the '(ddeESS [S+4])' buffer.  Therefore we create an
 ;;;     ESS process in the buffer as a placeholder and start a shell
 ;;;     in the ESS buffer.  From the shell we start Splus.  Once Splus
 ;;;     finishes initializing and kills the original shell, we start
@@ -132,24 +158,24 @@ connects it to the *S+4 ddeclient* window.")
 ;;;     inferior-ess-ddeclient, initialized to nil.  When there is a
 ;;;     non-nil value of inferior-ess-ddeclient we send lines to
 ;;;     inferior-ess-ddeclient rather than to the Splus process.
-;;; (2) There is no Splus process running in the *S+4 ddeclient*
+;;; (2) There is no Splus process running in the '(ddeESS [S+4])'
 ;;;     buffer.  Therefore inferior-ess will never see a prompt,
 ;;;     unless we first change it to the null prompt "^".  Then once
 ;;;     the process has started, we change it back.
 ;;; (3) When M-x S+4 starts Splus by a shell command, then Splus is an
-;;;     independent process and will be survive if the *S+4 ddeclient*
-;;;     buffer is killed (or emacs is quit).  The *S+4 ddeclient* is
+;;;     independent process and will be survive if the '(ddeESS [S+4])'
+;;;     buffer is killed (or emacs is quit).  The '(ddeESS [S+4])' is
 ;;;     made read-only and a warning is placed in it saying that "You
-;;;     can't type anything here."  Actually, if thestandalone Splus
-;;;     is killed and the *S+4 ddeclient* is made writable (C-x C-q),
-;;;     then *S+4 ddeclient* becomes a shell buffer.
+;;;     can't type anything here."  Actually, if the standalone Splus
+;;;     is killed and the '(ddeESS [S+4])' is made writable (C-x C-q),
+;;;     then '(ddeESS [S+4])' becomes a shell buffer.
 ;;;
 (defun S+4 (&optional proc-name)
   "Call 'S-PLUS 4.x', the 'GUI Thing' from StatSci.  Put S-Plus in an
-independent MS-Window (Splus persists even if the *S+4 ddeclient*
+independent MS-Window (Splus persists even if the '(ddeESS [S+4])'
 window is killed in emacs).  Do this by creating a comint process that
 calls sh.  Send a shell command in that sh buffer to call Splus.  When
-it completes set up a shell as a placeholder in the *S+4 ddeclient*
+it completes set up a shell as a placeholder in the '(ddeESS [S+4])'
 buffer.  The S-Plus options are correctly set.  In particular, the
 S-Plus Commands window is opened if the Options/General
 Settings/Startup menu says it should be.  There is a 30 second delay
@@ -167,12 +193,18 @@ is here to allow slow disks to start the Splus program."
 	  (append ess-customize-alist '((inferior-ess-primary-prompt   . "^"))))
     (setq ess-customize-alist		; change inferior-ess-start-args
 	  (append ess-customize-alist '((inferior-ess-start-args   . "-i"))))
-    (let ((s-proj (getenv "S_PROJ")))
-      (setenv "S_PROJ" (directory-file-name default-directory))
+    (let ((s-proj (getenv "S_PROJ"))
+	  (manpath (getenv "MANPATH")))
+      (cd (w32-short-file-name (directory-file-name default-directory)))
+      (setenv "S_PROJ" default-directory)
+      ;; I don't know why this PATH/MANPATH game is needed,
+      ;; except that it doesn't work without it.
+      (setenv "MANPATH" (getenv "PATH"))
       (inferior-ess)
       (sleep-for 2) ; need to wait, else working too fast!  The Splus
-		    ; command in *S+4 ddeclient* should follow the "$"
+		    ; command in '(ddeESS [S+4])' should follow the "$"
 		    ; prompt.  If not, then increase the sleep-for time!
+      (setenv "MANPATH" manpath)
       (setenv "S_PROJ" s-proj))
     (setq ess-customize-alist S+4-customize-alist)
     (ess-setq-vars-local ess-customize-alist)
@@ -187,18 +219,18 @@ is here to allow slow disks to start the Splus program."
     (goto-char (point-max))
     (insert (concat inferior-S+4-program-name " "
 		    inferior-ess-start-args)) ; Note: there is no final "&".
-    ;; Without the "&", the results of  !system.command  come to *S+4 ddeclient*
+    ;; Without the "&", the results of  !system.command  come to '(ddeESS [S+4])'
     ;; With the "&", the results of  !system.command  in S get lost.
     (inferior-ess-send-input)
     (sleep-for 30) ; Need to wait, else working too fast!
                    ; If the ess-current-process-name doesn't appear in the
        		   ; Splus Commands window increase the sleep-for time!
     (setq ess-local-process-name ess-current-process-name)
-    (ess-eval-visibly (concat "#" ess-current-process-name))
+    (ess-eval-linewise (concat "#" ess-current-process-name))
     (beginning-of-buffer)
     (insert
      "This is a placeholder buffer.  You can't type anything here.
-Use 'C-x b RET' to return to your file.\n
+Use `C-x b RET' to return to your file.\n
 Anything sent to this process from an S-mode buffer goes
 directly to the associated Splus Commands window.\n
 The S-Plus Commands window must be visible.
@@ -206,12 +238,14 @@ You may need to open the S-Plus Commands window manually (by clicking on
 Splus/Window/Commands Window).\n
 Any results of the   !system.command   typed at the S prompt in the
 Splus Commands window appear in this buffer.\n\n")
-    (goto-char (point-max))		; comint-mode-map makes *S+4 ddeclient*
+    (goto-char (point-max))		; comint-mode-map makes '(ddeESS [S+4])'
 ;;  (use-local-map comint-mode-map)     ;a shell buffer after Splus is finished.
     (set-buffer-process-coding-system 'raw-text-dos 'raw-text-unix)
     (toggle-read-only t)		; force buffer to be read-only
     (setq mode-name "ddeESS")
-    (ess-eval-visibly inferior-S+4-editor-pager-command)
+;;  (ess-eval-linewise inferior-S+4-editor-pager-command)
+    (if inferior-ess-language-start
+	(ess-eval-linewise inferior-ess-language-start))
     ))
 
 
@@ -220,7 +254,7 @@ Splus Commands window appear in this buffer.\n\n")
 (defun S+4-existing (&optional proc-name)
   "Call 'S-PLUS 4.x', the 'GUI Thing' from StatSci.  Do so by finding
 an existing S-Plus in an independent MS-Window (if there is one) and
-set up a *S+4 ddeclient* buffer in emacs.  If there is no existing
+set up a '(ddeESS [S+4])' buffer in emacs.  If there is no existing
 S-Plus, then a new one will be opened in the default directory,
 usually something like c:/Program Files/spls45se/users/yourname.
 If you have a HOME environment variable, it will open it there."
@@ -282,13 +316,15 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
     (insert "options(interactive=T)")
     (inferior-ess-send-input)
     (setq mode-name "iESS(Sqpe)")
-    (ess-eval-visibly inferior-S+4-editor-pager-command)
+;;  (ess-eval-linewise inferior-S+4-editor-pager-command)
+    (if inferior-ess-language-start
+	(ess-eval-linewise inferior-ess-language-start))
     (if shome-nil-p (setenv "SHOME" nil))))
 
 
 
 (defun S+4-mode (&optional proc-name)
-  "Major mode for editing S+4 source.  See ess-mode for more help."
+  "Major mode for editing S+4 source.  See `ess-mode' for more help."
   (interactive)
   (setq ess-customize-alist S+4-customize-alist)
   (ess-mode S+4-customize-alist proc-name)
@@ -303,10 +339,10 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
 
 (defun S+4-msdos (&optional proc-name)
   "Call 'S-PLUS 4.x', the 'GUI Thing' from StatSci.  Put S-Plus in an
-independent MS-Window (Splus persists even if the *S+4 ddeclient*
+independent MS-Window (Splus persists even if the '(ddeESS [S+4])'
 window is killed in emacs).  Do this by creating a comint process that
 calls sh.  Send a shell command in that sh buffer to call Splus.  When
-it completes set up a shell as a placeholder in the *S+4 ddeclient*
+it completes set up a shell as a placeholder in the '(ddeESS [S+4])'
 buffer.  The S-Plus options are correctly set.  In particular, the
 S-Plus Commands window is opened if the Options/General
 Settings/Startup menu says it should be.  There is a 30 second delay
@@ -326,10 +362,11 @@ is here to allow slow disks to start the Splus program."
     (setq ess-customize-alist		; change inferior-ess-start-args
 	  (append ess-customize-alist '((inferior-ess-start-args   . ""))))
     (let ((s-proj (getenv "S_PROJ")))
-      (setenv "S_PROJ" (directory-file-name default-directory))
+      (cd (w32-short-file-name (directory-file-name default-directory)))
+      (setenv "S_PROJ" default-directory)
       (inferior-ess)
       (sleep-for 2) ; need to wait, else working too fast!  The Splus
-		    ; command in *S+4 ddeclient* should follow the "$"
+		    ; command in '(ddeESS [S+4])' should follow the "$"
 		    ; prompt.  If not, then increase the sleep-for time!
       (setenv "S_PROJ" s-proj))
     (setq ess-customize-alist S+4-customize-alist)
@@ -342,10 +379,11 @@ is here to allow slow disks to start the Splus program."
 ;;; end of what belongs in customize-alist
     (setq comint-input-sender 'comint-simple-send)
     (setq comint-process-echoes nil)
+    (set-buffer-process-coding-system 'raw-text-dos 'raw-text-dos)
     (goto-char (point-max))
     (insert (concat inferior-S+4-program-name " "
 		    inferior-ess-start-args)) ; Note: there is no final "&".
-; Without the "&", the results of  !system.command  come to *S+4 ddeclient*
+; Without the "&", the results of  !system.command  come to '(ddeESS [S+4])'
 ; With the "&", the results of  !system.command  in S get lost.
     (inferior-ess-send-input)
     (sleep-for 30) ; Need to wait, else working too fast!
@@ -356,7 +394,7 @@ is here to allow slow disks to start the Splus program."
   (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m nil t)
 ;;; end from msdos-minor-mode
     (setq ess-local-process-name ess-current-process-name)
-    (ess-eval-visibly (concat "#" ess-current-process-name))
+    (ess-eval-linewise (concat "#" ess-current-process-name))
     (beginning-of-buffer)
     (insert
      "This is a placeholder buffer.  You can't type anything here.
@@ -370,22 +408,24 @@ There is a 30 second delay when this program starts during which the
 emacs screen will be partially blank.\n
 Remember to
 `q()' from S-Plus and
- then M-x C-q exit from the `*S+4 ddeclient* buffer,
+ then C-x C-q exit from the `'(ddeESS [S+4])'' buffer,
 or take the risk of not being able to shut down your computer
 and suffering through scandisk.\n
 Any results of the   !system.command   typed at the S prompt in the
 Splus Commands window (are supposed to) appear in this buffer.\n\n")
-    (goto-char (point-max))	       ; comint-mode-map makes *S+4 ddeclient*
+    (goto-char (point-max))	       ; comint-mode-map makes '(ddeESS [S+4])'
     (use-local-map comint-mode-map)    ; a shell buffer after Splus is finished.
     (toggle-read-only t)	       ; force buffer to be read-only
     (setq mode-name "ddeESS")
-    (ess-eval-visibly inferior-S+4-editor-pager-command)
+;;  (ess-eval-linewise inferior-S+4-editor-pager-command)
+    (if inferior-ess-language-start
+      (ess-eval-linewise inferior-ess-language-start))
     ))
 
 (defun S+4-msdos-existing (&optional proc-name)
   "Call 'S-PLUS 4.x', the 'GUI Thing' from StatSci.  Do so by finding
 an existing S-Plus in an independent MS-Window (if there is one) and
-set up a *S+4 ddeclient* buffer in emacs.  If there is no existing
+set up a '(ddeESS [S+4])' buffer in emacs.  If there is no existing
 S-Plus, then a new one will be opened in the default directory,
 usually something like c:/Program Files/spls45se/users/yourname.
 If you have a HOME environment variable, it will open it there."
