@@ -1,6 +1,14 @@
 ;; essddr.el --- Support for editing R documentation (Rd) source
 
-;;; Copyright (C) 1998 KH <Kurt.Hornik@ci.tuwien.ac.at>
+;;; Copyright (C) 1998--2001 KH <Kurt.Hornik@ci.tuwien.ac.at>, AJR, MM
+;;;
+
+;; Author: KH <Kurt.Hornik@ci.tuwien.ac.at>
+;; Maintainer: A.J. Rossini <rossini@biostat.washington.edu>
+;; Created: 25 July 1997
+;; Modified: $Date: 2002/01/28 08:35:20 $
+;; Version: $Revision: 5.19 $
+;; RCS: $Id: essddr.el,v 5.19 2002/01/28 08:35:20 maechler Exp $
 
 ;; This file is part of ESS (Emacs Speaks Statistics).
 
@@ -19,24 +27,45 @@
 ;; obtain it by writing to the Free Software Foundation, Inc., 675 Mass
 ;; Ave, Cambridge, MA 02139, USA.
 
-;;; ESS RCS: $Id: essddr.el,v 5.6 1999/01/04 13:52:43 maechler Exp $
+;;; ESS RCS: $Id: essddr.el,v 5.19 2002/01/28 08:35:20 maechler Exp $
 
 ;;; Code:
 
 ;; To stave off byte compiler errors
 (eval-when-compile (require 'ess-help))
 
-(defvar essddr-version "0.1.8"
+(defvar essddr-version "0.9"
   "Current version of essddr.el.")
 
 (defvar essddr-maintainer-address
   "Kurt Hornik <Kurt.Hornik@ci.tuwien.ac.at>"
   "Current maintainer of essddr.el.")
 
-(autoload 'ess-eval-region             "ess-mode" "[autoload]" t)
-(autoload 'ess-eval-line-and-next-line "ess-mode" "[autoload]" t)
-(autoload 'ess-nuke-help-bs            "ess-help" "[autoload]" t)
-(autoload 'ess-help-mode               "ess-help" "[autoload]" t)
+;; Special support for XEmacs (curtesy of auctex):
+
+(when (featurep 'xemacs)
+
+  (defun Rd-active-mark ()
+    (and zmacs-regions (mark)))
+  )
+
+;; Special support for GNU Emacs
+
+(unless (featurep 'xemacs)
+
+  (defun Rd-active-mark ()
+    (and transient-mark-mode mark-active))
+  )
+
+
+(autoload 'ess-eval-region		"ess-inf" "[autoload]" t)
+(autoload 'ess-eval-line-and-step	"ess-inf" "[autoload]" t)
+(autoload 'ess-switch-process		"ess-inf" "[autoload]" t)
+(autoload 'ess-switch-to-ESS		"ess-inf" "[autoload]" t)
+(autoload 'ess-switch-to-end-of-ESS	"ess-inf" "[autoload]" t)
+
+(autoload 'ess-help-mode		"ess-help" "[autoload]" t)
+(autoload 'ess-nuke-help-bs		"ess-help" "[autoload]" t)
 
 (defvar Rd-mode-abbrev-table nil
   "Abbrev table for R documentation keywords.
@@ -46,25 +75,29 @@ All Rd mode abbrevs start with a grave accent (`).")
   (define-abbrev-table 'Rd-mode-abbrev-table ())
   (define-abbrev Rd-mode-abbrev-table "`ag" "\\arguments")
   (define-abbrev Rd-mode-abbrev-table "`al" "\\alias")
+  (define-abbrev Rd-mode-abbrev-table "`au" "\\author")
   (define-abbrev Rd-mode-abbrev-table "`bf" "\\bold")
   (define-abbrev Rd-mode-abbrev-table "`co" "\\code")
   (define-abbrev Rd-mode-abbrev-table "`de" "\\describe")
   (define-abbrev Rd-mode-abbrev-table "`dn" "\\description")
   (define-abbrev Rd-mode-abbrev-table "`dt" "\\details")
-  (define-abbrev Rd-mode-abbrev-table "`ex" "\\examples")
   (define-abbrev Rd-mode-abbrev-table "`em" "\\emph")
-  (define-abbrev Rd-mode-abbrev-table "`em" "\\enumerate")
+  (define-abbrev Rd-mode-abbrev-table "`en" "\\enumerate")
+  (define-abbrev Rd-mode-abbrev-table "`ex" "\\examples")
   (define-abbrev Rd-mode-abbrev-table "`fi" "\\file")
   (define-abbrev Rd-mode-abbrev-table "`fi" "\\format")
   (define-abbrev Rd-mode-abbrev-table "`it" "\\item")
   (define-abbrev Rd-mode-abbrev-table "`iz" "\\itemize")
   (define-abbrev Rd-mode-abbrev-table "`kw" "\\keyword")
   (define-abbrev Rd-mode-abbrev-table "`li" "\\link")
+  (define-abbrev Rd-mode-abbrev-table "`me" "\\method")
   (define-abbrev Rd-mode-abbrev-table "`na" "\\name")
+  (define-abbrev Rd-mode-abbrev-table "`no" "\\note")
   (define-abbrev Rd-mode-abbrev-table "`re" "\\references")
   (define-abbrev Rd-mode-abbrev-table "`sa" "\\seealso")
   (define-abbrev Rd-mode-abbrev-table "`se" "\\section")
-  (define-abbrev Rd-mode-abbrev-table "`se" "\\source")
+  (define-abbrev Rd-mode-abbrev-table "`so" "\\source")
+  (define-abbrev Rd-mode-abbrev-table "`sy" "\\synopsis")
   (define-abbrev Rd-mode-abbrev-table "`ta" "\\tabular")
   (define-abbrev Rd-mode-abbrev-table "`ti" "\\title")
   (define-abbrev Rd-mode-abbrev-table "`us" "\\usage")
@@ -109,14 +142,19 @@ All Rd mode abbrevs start with a grave accent (`).")
 (defvar Rd-section-names
   '("arguments" "alias" "author" "describe" "description" "details"
     "enumerate" "examples" "format" "itemize" "keyword" "name" "note"
-    "references" "seealso" "section" "source" "tabular" "title" "usage"
-    "value" "verbatim"))
+    "references" "seealso" "section" "source" "synopsis" "tabular"
+    "title" "usage" "value" "verbatim"))
 (defvar Rd-keywords
   '("Alpha" "Gamma" "R" "alpha" "beta" "bold" "cr" "code" "deqn" "dots"
     "email" "emph" "epsilon" "eqn" "file" "ge" "item" "lambda" "ldots"
-    "le" "left" "link" "mu" "pi" "right" "tab" "sigma" "url"))
+    "le" "left" "link" "method" "mu" "pi" "right" "tab" "sigma" "url"))
 
-(defvar Rd-bold-face 'bold)
+;; Need to fix Rd-bold-face problem.
+;;
+;; (defvar Rd-bold-face 'bold)
+;(defvar Rd-bold-face nil)
+;(make-face Rd-bold-face "R documentation bold face")
+;(make-face-bold Rd-bold-face
 
 (defvar Rd-font-lock-keywords
   (list
@@ -124,15 +162,19 @@ All Rd mode abbrevs start with a grave accent (`).")
     (concat "\\\\\\("
 	    (mapconcat 'identity Rd-section-names "\\|")
 	    "\\>\\)")
-    'Rd-bold-face)
+    'font-lock-reference-face) ; Rd-bold-face
    (cons
     (concat "\\\\\\("
 	    (mapconcat 'identity Rd-keywords "\\|")
 	    "\\>\\)")
-    'font-lock-keyword-face))
+    'font-lock-keyword-face)
+   '("^#\\(ifn?def\\)\\s-+\\(\\sw+\\)"
+    (1 font-lock-builtin-face)
+    (2 font-lock-variable-name-face nil t))
+   '("^#\\(endif\\)" 1 font-lock-builtin-face))
   "Additional Rd expressions to highlight.")
 
-(defvar Rd-indent-level 4
+(defvar Rd-indent-level 2
   "*Indentation of Rd code with respect to containing blocks.")
 
 (defvar Rd-mode-map nil
@@ -145,25 +187,34 @@ All Rd mode abbrevs start with a grave accent (`).")
     (define-key map "\C-m" 'reindent-then-newline-and-indent)
     (define-key map "\C-c\C-p" 'Rd-preview-help)
     (define-key map "\C-c\C-j" 'Rd-mode-insert-item)
-    (define-key map "\C-c\C-s" 'Rd-mode-insert-section)
     (define-key map "\C-c\C-e" 'Rd-mode-insert-skeleton)
+    (define-key map "\C-c\C-f" 'Rd-font)
+    ;;  ^C^F ^E : \emph{ . }
+    ;;  ^C^F ^C : \code{ . }
+    ;;  ^C^F ^L : \code{\link{ . }}
+    (define-key map "\C-c\C-s" 'Rd-mode-insert-section)
+    (define-key map "\C-c\C-n" 'ess-eval-line-and-step)
     (define-key map "\C-c\C-r" 'ess-eval-region)
-    (define-key map "\C-c\C-n" 'ess-eval-line-and-next-line)
+    (define-key map "\C-c\C-v" 'ess-display-help-on-object)
+    (define-key map "\C-c\C-w" 'ess-switch-process); is on C-c C-s in ess-mode..
     (define-key map "\C-c\C-y" 'ess-switch-to-ESS)
     (define-key map "\C-c\C-z" 'ess-switch-to-end-of-ESS)
     (setq Rd-mode-map map)))
 
 (defvar Rd-mode-menu
   (list "Rd"
+	["Markup [word]"		Rd-font t]
 	["Insert Item"			Rd-mode-insert-item t]
 	["Insert Section"		Rd-mode-insert-section t]
 	["Insert Skeleton"		Rd-mode-insert-skeleton t]
 	"-"
 	["Preview"			Rd-preview-help t]
 	"-"
-	["Eval Line"			ess-eval-line-and-next-line t]
+	["Eval Line"			ess-eval-line-and-step t]
 	["Eval Region"			ess-eval-region t]
-	["Switch to Process"		ess-switch-to-ESS t]
+	["Switch to ESS Process"	ess-switch-to-ESS t]
+	["Switch the ESS Process"	ess-switch-process t]
+	["Switch to end{ESS Pr}"	ess-switch-to-end-of-ESS t]
 	"-"
 	["Toggle Abbrev Mode"		abbrev-mode t]
 	["Toggle Auto-Fill Mode"	auto-fill-mode t]
@@ -198,7 +249,7 @@ Keybindings
 Variables you can use to customize Rd mode
 ==========================================
 
-Rd-indent-level
+`Rd-indent-level'
   Indentation of Rd code with respect to containing blocks.
   Default is 4.
 
@@ -208,10 +259,10 @@ To automatically turn on the abbrev and font-lock features, add the
 following lines to your `.emacs' file:
 
   (add-hook 'Rd-mode-hook
-            (lambda ()
-              (abbrev-mode 1)
-              (if (eq window-system 'x)
-                  (font-lock-mode 1))))"
+	    (lambda ()
+	      (abbrev-mode 1)
+	      (if (eq window-system 'x)
+		  (font-lock-mode 1))))"
 
   (interactive)
   (text-mode)
@@ -240,6 +291,7 @@ following lines to your `.emacs' file:
   (message "Rd mode version %s" essddr-version)
   (run-hooks 'Rd-mode-hook))
 
+;; FIXME: The following should be moved to ess-utils.el, no? (MM thinks)
 (defun ess-point (position)
   "Returns the value of point at certain positions."
   (save-excursion
@@ -260,7 +312,8 @@ following lines to your `.emacs' file:
 (defun Rd-mode-in-verbatim-p ()
   (let ((pos (point)))
     (save-excursion
-      (if (and (re-search-backward "\\\\\\(usage\\|examples\\)" nil t)
+      (if (and (re-search-backward
+		"\\\\\\(usage\\|examples\\|synopsis\\)" nil t)
 	       (re-search-forward "\\s(" nil t))
 	  (condition-case ()
 	      (progn
@@ -269,13 +322,24 @@ following lines to your `.emacs' file:
 	    (error t))
 	nil))))
 
+(defun Rd-mode-in-preprocessor-line-p ()
+  (save-excursion
+    (beginning-of-line)
+    (looking-at "[ \t]*#\\(ifdef\\|endif\\)")))
+
 (defun Rd-mode-calculate-indent ()
   "Return appropriate indentation for current line in Rd mode."
   (interactive)
   (save-excursion
     (beginning-of-line)
-    (if (Rd-mode-in-verbatim-p)
-	nil				; Don't do anything in verbatims
+    (cond
+     ((Rd-mode-in-verbatim-p)
+      ;; Don't do anything in verbatims
+      nil)
+     ((Rd-mode-in-preprocessor-line-p)
+      ;; Indent to 0
+      0)
+     (t
       (let ((p (progn
 		 (re-search-forward "[ \t]*\\s)*" (ess-point 'eol) t)
 		 (point))))
@@ -283,7 +347,8 @@ following lines to your `.emacs' file:
 		(Rd-mode-in-verbatim-p))
 	    0
 	  (set-syntax-table Rd-mode-parse-syntax-table)
-	  (while (and (looking-at "[ \t]*$")
+	  (while (and (or (looking-at "[ \t]*$")
+			  (Rd-mode-in-preprocessor-line-p))
 		      (not (bobp)))
 	    (forward-line -1))
 	  (re-search-forward "[ \t]*\\s)*" (ess-point 'eol) t)
@@ -291,7 +356,7 @@ following lines to your `.emacs' file:
 	      (+ (current-indentation)
 		 (* (car (parse-partial-sexp (point) p))
 		    Rd-indent-level))
-	    (set-syntax-table Rd-mode-syntax-table)))))))
+	    (set-syntax-table Rd-mode-syntax-table))))))))
 
 (defun Rd-mode-indent-line ()
   "Indent current line as Rd source."
@@ -326,8 +391,8 @@ following lines to your `.emacs' file:
   (insert "\\name{}\n")
   (insert "\\alias{}\n")
   (insert "\\title{}\n")
-  (insert "\\usage{\n}\n")
   (insert "\\description{\n}\n")
+  (insert "\\usage{\n}\n")
   (insert "\\arguments{\n}\n")
   (insert "\\value{\n}\n")
   (insert "\\details{\n}\n")
@@ -337,6 +402,70 @@ following lines to your `.emacs' file:
   (insert "\\author{}\n")
   (insert "\\keyword{}"))
 
+;; This is an `easy' version of (defun TeX-font ..) in AUCtex's  tex.el ;
+;;  see TeX-font-list and also LaTeX-font-list in latex.el
+
+(defun Rd-font (what)
+  "Insert template for font command.
+ WHAT determines the font to use, as specified by `Rd-font-list'."
+  (interactive "c")
+  ;;TeX had : (Rd-update-style)
+  (let* ((entry (assoc what Rd-font-list))
+	 (before (nth 1 entry))
+	 (after (nth 2 entry)))
+    (cond ((null entry) ;; help on possibilities :
+	   (let ((help
+		  (concat
+		   "Rd Markup (available from C-c C-f):\n\n\t"
+		   "KEY          Rd-Markup\n\n"
+		   (mapconcat
+		    '(lambda (entry)
+		       ;; A textual description of an ENTRY in TeX-font-list.
+		       (concat (format "%11s  "
+				       (key-description
+					(char-to-string (nth 0 entry))))
+			       (format "%14s %-3s"
+				       (nth 1 entry) (nth 2 entry))))
+		    Rd-font-list "\n"))))
+	     (with-output-to-temp-buffer "*Help*"
+	       (set-buffer "*Help*")
+	       (insert help))))
+
+	  ((Rd-active-mark)
+	   (save-excursion
+	     (cond ((> (mark) (point))
+		    (insert before)
+		    (goto-char (mark))
+		    (insert after))
+		   (t
+		    (insert after)
+		    (goto-char (mark))
+		    (insert before)))))
+	  (t
+	   (insert before)
+	   (save-excursion
+	     (insert after))))))
+
+
+(defvar Rd-font-list
+  '((?\C-b "\\bold{"	"}")
+    (?\C-c "\\code{"	"}")
+    (?\C-e "\\emph{"	"}")
+    (?\C-l "\\link{"	"}")
+    (?l "\\code{\\link{" "}}")
+    (?\C-m "\\email{"	"}")
+    (?\C-q "\\eqn{"	"}")
+    (?\C-u "\\url{"	"}")
+    )
+  "List of ``fonts'' used by Rd-font.
+
+Each entry is a list.
+The first element is the key to activate the font.
+The second element is the string to insert before point, and the third
+element is the string to insert after point."
+)
+
+
 (defun Rd-preview-help ()
   (interactive)
   (require 'ess-help)
