@@ -1,14 +1,14 @@
 ;;; essl-s.el --- Support for editing S source code
 
-;; Copyright (C) 1989-1997 Bates, Kademan, Ritter, Smith, Hornik,
-;; Heiberger, Maechler, and Rossini.
+;; Copyright (C) 1989-2001 D. Bates, Kademan, Ritter, D.M. Smith, K. Hornik,
+;; R.M. Heiberger, M. Maechler, and A.J. Rossini.
 
-;; Author: A.J. Rossini <rossini@stat.sc.edu>
-;; Maintainer: A.J. Rossini <rossinI@stat.sc.edu>
+;; Author: A.J. Rossini <rossini@biostat.washington.edu>
+;; Maintainer: A.J. Rossini <rossini@biostat.washington.edu>
 ;; Created: 26 Aug 1997
-;; Modified: $Date: 1998/11/11 12:44:49 $
-;; Version: $Revision: 5.11 $
-;; RCS: $Id: essl-s.el,v 5.11 1998/11/11 12:44:49 maechler Exp $
+;; Modified: $Date: 2002/01/15 08:47:17 $
+;; Version: $Revision: 5.27 $
+;; RCS: $Id: essl-s.el,v 5.27 2002/01/15 08:47:17 maechler Exp $
 
 ;; This file is part of ESS (Emacs Speaks Statistics).
 
@@ -34,10 +34,150 @@
 
  ; Requires and autoloads
 
+(ess-message "[essl-s:] (def** ) only ...")
 
  ; Configuration variables
 
+(defvar S-syntax-table nil "Syntax table for S code.")
+(if S-syntax-table
+    nil
+  (setq S-syntax-table (make-syntax-table))
+  (modify-syntax-entry ?\\ "\\" S-syntax-table)
+  (modify-syntax-entry ?+  "."  S-syntax-table)
+  (modify-syntax-entry ?-  "."  S-syntax-table)
+  (modify-syntax-entry ?=  "."  S-syntax-table)
+  (modify-syntax-entry ?%  "."  S-syntax-table)
+  (modify-syntax-entry ?<  "."  S-syntax-table)
+  (modify-syntax-entry ?>  "."  S-syntax-table)
+  (modify-syntax-entry ?&  "."  S-syntax-table)
+  (modify-syntax-entry ?|  "."  S-syntax-table)
+  (modify-syntax-entry ?\' "\"" S-syntax-table)
+  (modify-syntax-entry ?#  "<"  S-syntax-table) ; open comment
+  (modify-syntax-entry ?\n ">"  S-syntax-table) ; close comment
+  ;;(modify-syntax-entry ?.  "w"  S-syntax-table) ; "." used in S obj names
+  (modify-syntax-entry ?.  "_"  S-syntax-table) ; see above/below,
+					; plus consider separation.
+  (modify-syntax-entry ?$  "_"  S-syntax-table) ; foo.bar$hack is 1 symbol
+  (modify-syntax-entry ?_  "."  S-syntax-table)
+  (modify-syntax-entry ?*  "."  S-syntax-table)
+  (modify-syntax-entry ?<  "."  S-syntax-table)
+  (modify-syntax-entry ?>  "."  S-syntax-table)
+  (modify-syntax-entry ?/  "."  S-syntax-table))
 
+(defvar S-editing-alist
+  '((paragraph-start              . (concat "^$\\|" page-delimiter))
+    (paragraph-separate           . (concat "^$\\|" page-delimiter))
+    (paragraph-ignore-fill-prefix . t)
+    (require-final-newline        . t)
+    (comment-start                . "#")
+    (comment-start-skip           . "#+ *")
+    (comment-column               . 40)
+    ;;(comment-indent-function  . 'S-comment-indent)
+    ;;(ess-comment-indent           . 'S-comment-indent)
+    ;;(ess-indent-line                      . 'S-indent-line)
+    ;;(ess-calculate-indent           . 'S-calculate-indent)
+    (indent-line-function            . 'S-indent-line)
+    (parse-sexp-ignore-comments   . t)
+    (ess-set-style                . ess-default-style)
+    (ess-local-process-name       . nil)
+    ;;(ess-keep-dump-files          . 'ask)
+    (ess-mode-syntax-table        . S-syntax-table)
+    ;; For Changelog add, require ' ' before <- : "attr<-" is a function name :
+    (add-log-current-defun-header-regexp . "^\\(.+\\)\\s-+<-[ \t\n]*function")
+    (font-lock-defaults           . '(ess-mode-font-lock-keywords
+				      nil nil ((?\. . "w")))))
+  "General options for editing S, S+, and R source files.")
+
+;;; Changes from S to S-PLUS 3.x.  (standard S3 should be in essl-s!).
+
+(defconst S+-help-sec-keys-alist
+  '((?a . "ARGUMENTS:")
+    (?b . "BACKGROUND:")
+    (?B . "BUGS:")
+    (?d . "DESCRIPTION:")
+    (?D . "DETAILS:")
+    (?e . "EXAMPLES:")
+    (?n . "NOTE:")
+    (?O . "OPTIONAL ARGUMENTS:")
+    (?R . "REQUIRED ARGUMENTS:")
+    (?r . "REFERENCES:")
+    (?s . "SEE ALSO:")
+    (?S . "SIDE EFFECTS:")
+    (?u . "USAGE:")
+    (?v . "VALUE:"))
+  "Alist of (key . string) pairs for use in section searching.")
+;;; `key' indicates the keystroke to use to search for the section heading
+;;; `string' in an S help file. `string' is used as part of a
+;;; regexp-search, and so specials should be quoted.
+
+;; S ver.3 (NOT S-Plus)
+(defconst S3-help-sec-keys-alist
+  '((?a . "ARGUMENTS:")
+    (?b . "BACKGROUND:")
+    (?B . "BUGS:")
+    (?d . "DESCRIPTION:")
+    (?D . "DETAILS:")
+    (?e . "EXAMPLES:")
+    (?n . "NOTE:")
+    (?r . "REFERENCES:")
+    (?s . "SEE ALSO:")
+    (?S . "SIDE EFFECTS:")
+    (?u . "USAGE:")
+    (?v . "VALUE:"))
+  "Help section keys for S ver.3.")
+
+;; S ver.4 (NOT S-Plus)
+(defconst S4-help-sec-keys-alist
+  '((?a . "ARGUMENTS:")
+    (?b . "BACKGROUND:")
+    (?B . "BUGS:")
+    (?d . "DESCRIPTION:")
+    (?D . "DETAILS:")
+    (?e . "EXAMPLES:")
+    (?n . "NOTE:")
+    (?r . "REFERENCES:")
+    (?s . "SEE ALSO:")
+    (?S . "SIDE EFFECTS:")
+    (?u . "USAGE:")
+    (?v . "VALUE:"))
+  "Help section keys for S4.")
+
+;; R
+(defconst R-help-sec-keys-alist
+  '((?a . "\\s *Arguments:")
+    (?d . "\\s *Description:")
+    (?D . "\\s *Details:")
+    (?e . "\\s *Examples:")
+    (?n . "\\s *Note:")
+    (?r . "\\s *References:")
+    (?s . "\\s *See Also:")
+    (?u . "\\s *Usage:")
+    (?v . "\\s *Value[s]?")	;
+    )
+  "Alist of (key . string) pairs for use in help section searching.")
+
+
+(defconst ess-help-S+-sec-regex "^[A-Z. ---]+:$"
+  "Reg(ular) Ex(pression) of section headers in help file.")
+
+(defconst ess-help-R-sec-regex "^\\s *[A-Z[a-z. ---]+:$"
+  "Reg(ular) Ex(pression) of section headers in help file.")
+
+;;; S-mode extras of Martin Maechler, Statistik, ETH Zurich.
+;;; See also ./ess-utils.el
+
+(defvar ess-function-outline-file
+  (concat ess-lisp-directory "/../etc/" "function-outline.S")
+  "The file name of the ess-function outline that is to be inserted at point,
+when \\<ess-mode-map>\\[ess-insert-function-outline] is used.
+Placeholders (substituted `at runtime'): $A$ for `Author', $D$ for `Date'.")
+
+;; Use the user's own ~/S/emacs-fun.outline  is (s)he has one : ---
+(let ((outline-file (concat (getenv "HOME") "/S/function-outline.S")))
+  (if (file-exists-p outline-file)
+      (setq ess-function-outline-file outline-file)))
+
+ ; Function Definitions
 
 (defun S-comment-indent ()
   "Indentation for S comments."
@@ -97,7 +237,6 @@ Return the amount the indentation changed by."
 	  (goto-char (- (point-max) pos))))
     shift-amt))
 
-
 (defun S-calculate-indent (&optional parse-start)
   "Return appropriate indentation for current line as S code.
 In usual case returns an integer: the column to indent to.
@@ -144,7 +283,10 @@ Returns nil if line starts inside a string, t if in a comment."
 		      ;; End
 		      (+ (current-column) ess-expression-offset))
 		     ((and (numberp ess-arg-function-offset)
-			   (re-search-backward "=[ \t]*\\s\"*\\(\\w\\|\\s_\\)+\\s\"*[ \t]*" bol t))
+			   (re-search-backward
+			    "=[ \t]*\\s\"*\\(\\w\\|\\s_\\)+\\s\"*[ \t]*"
+			    bol
+			    t))
 		      (forward-sexp -1)
 		      (+ (current-column) ess-arg-function-offset))
 		     ;; "expression" is searched before "=".
@@ -195,8 +337,9 @@ Returns nil if line starts inside a string, t if in a comment."
 		 ;; If no previous statement,
 		 ;; indent it relative to line brace is on.
 		 ;; For open brace in column zero, don't let statement
-		 ;; start there too.  If ess-indent-level is zero,
-		 ;; use ess-brace-offset + ess-continued-statement-offset instead.
+		 ;; start there too.  If ess-indent-level is zero, use
+		 ;; ess-brace-offset + ess-continued-statement-offset
+		 ;; instead.
 		 ;; For open-braces not the first thing in a line,
 		 ;; add in ess-brace-imaginary-offset.
 		 (+ (if (and (bolp) (zerop ess-indent-level))
@@ -216,152 +359,9 @@ Returns nil if line starts inside a string, t if in a comment."
 		      ;; Get initial indentation of the line we are on.
 		      (current-indentation))))))))))
 
-
-
-
-(defvar S-syntax-table nil "Syntax table for S code.")
-(if S-syntax-table
-    nil
-  (setq S-syntax-table (make-syntax-table))
-  (modify-syntax-entry ?\\ "\\" S-syntax-table)
-  (modify-syntax-entry ?+  "."  S-syntax-table)
-  (modify-syntax-entry ?-  "."  S-syntax-table)
-  (modify-syntax-entry ?=  "."  S-syntax-table)
-  (modify-syntax-entry ?%  "."  S-syntax-table)
-  (modify-syntax-entry ?<  "."  S-syntax-table)
-  (modify-syntax-entry ?>  "."  S-syntax-table)
-  (modify-syntax-entry ?&  "."  S-syntax-table)
-  (modify-syntax-entry ?|  "."  S-syntax-table)
-  (modify-syntax-entry ?\' "\"" S-syntax-table)
-  (modify-syntax-entry ?#  "<"  S-syntax-table) ; open comment
-  (modify-syntax-entry ?\n ">"  S-syntax-table) ; close comment
-  ;;(modify-syntax-entry ?.  "w"  S-syntax-table) ; "." used in S obj names
-  (modify-syntax-entry ?.  "_"  S-syntax-table) ; see above/below,
-					; plus consider separation.
-  (modify-syntax-entry ?$  "_"  S-syntax-table) ; foo.bar$hack is 1 symbol
-  (modify-syntax-entry ?_  "."  S-syntax-table)
-  (modify-syntax-entry ?*  "."  S-syntax-table)
-  (modify-syntax-entry ?<  "."  S-syntax-table)
-  (modify-syntax-entry ?>  "."  S-syntax-table)
-  (modify-syntax-entry ?/  "."  S-syntax-table))
-
-
-(defvar S-editing-alist
-  '((paragraph-start              . (concat "^$\\|" page-delimiter))
-    (paragraph-separate           . (concat "^$\\|" page-delimiter))
-    (paragraph-ignore-fill-prefix . t)
-    (require-final-newline        . t)
-    (comment-start                . "#")
-    (comment-start-skip           . "#+ *")
-    (comment-column               . 40)
-    ;;(comment-indent-function  . 'S-comment-indent)
-    ;;(ess-comment-indent           . 'S-comment-indent)
-    ;;(ess-indent-line                      . 'S-indent-line)
-    ;;(ess-calculate-indent           . 'S-calculate-indent)
-    (indent-line-function            . 'S-indent-line)
-    (parse-sexp-ignore-comments   . t)
-    (ess-set-style                . ess-default-style)
-    (ess-local-process-name       . nil)
-    ;;(ess-keep-dump-files          . 'ask)
-    (ess-mode-syntax-table        . S-syntax-table)
-    (font-lock-defaults           . '(ess-mode-font-lock-keywords
-				      nil nil ((?\. . "w")))))
-  "General options for editing S, S+, and R source files.")
-
-
-;;; Changes from S to S-PLUS 3.x.  (standard S3 should be in essl-s!).
-
-(defconst S+-help-sec-keys-alist
-  '((?a . "ARGUMENTS:")
-    (?b . "BACKGROUND:")
-    (?B . "BUGS:")
-    (?d . "DESCRIPTION:")
-    (?D . "DETAILS:")
-    (?e . "EXAMPLES:")
-    (?n . "NOTE:")
-    (?O . "OPTIONAL ARGUMENTS:")
-    (?R . "REQUIRED ARGUMENTS:")
-    (?r . "REFERENCES:")
-    (?s . "SEE ALSO:")
-    (?S . "SIDE EFFECTS:")
-    (?u . "USAGE:")
-    (?v . "VALUE:"))
-  "Alist of (key . string) pairs for use in section searching.")
-;;; `key' indicates the keystroke to use to search for the section heading
-;;; `string' in an S help file. `string' is used as part of a
-;;; regexp-search, and so specials should be quoted.
-
-
-;; S ver.3 (NOT S-Plus)
-(defconst S3-help-sec-keys-alist
-  '((?a . "ARGUMENTS:")
-    (?b . "BACKGROUND:")
-    (?B . "BUGS:")
-    (?d . "DESCRIPTION:")
-    (?D . "DETAILS:")
-    (?e . "EXAMPLES:")
-    (?n . "NOTE:")
-    (?r . "REFERENCES:")
-    (?s . "SEE ALSO:")
-    (?S . "SIDE EFFECTS:")
-    (?u . "USAGE:")
-    (?v . "VALUE:"))
-  "Help section keys for S ver.3.")
-
-
-(defconst S4-help-sec-keys-alist
-  '((?a . "ARGUMENTS:")
-    (?b . "BACKGROUND:")
-    (?B . "BUGS:")
-    (?d . "DESCRIPTION:")
-    (?D . "DETAILS:")
-    (?e . "EXAMPLES:")
-    (?n . "NOTE:")
-    (?r . "REFERENCES:")
-    (?s . "SEE ALSO:")
-    (?S . "SIDE EFFECTS:")
-    (?u . "USAGE:")
-    (?v . "VALUE:"))
-  "Help section keys for S4.")
-
-(defconst R-help-sec-keys-alist
-  '((?a . "\\s *Arguments:")
-    (?d . "\\s *Description:")
-    (?e . "\\s *Examples:")
-    (?n . "\\s *Note:")
-    (?r . "\\s *References:")
-    (?s . "\\s *See Also:")
-    (?v . "\\s *Value[s]?")	;
-    )) ;; "Alist of (key . string) pairs for use in section searching."
-
-(defconst ess-help-S+-sec-regex "^[A-Z. ---]+:$"
-  "Reg(ular) Ex(pression) of section headers in help file")
-
-(defconst ess-help-R-sec-regex "^\\s *[A-Z[a-z. ---]+:$")
-
-
-;;; S4 stuff.
-;;            --> moved to essd-s4.el
-
-
-;;;    S-mode extras of Martin Maechler, Statistik, ETH Zurich.
-
-;;>> Moved things into --> ./ess-utils.el
-
-(defvar ess-function-outline-file
-  (concat ess-lisp-directory "/../etc/" "function-outline.S")
-  "The file name of the ess-function outline that is to be inserted at point,
-when \\<ess-mode-map>\\[ess-insert-function-outline] is used.
-Placeholders (substituted `at runtime'): $A$ for `Author', $D$ for `Date'.")
-
-;; Use the user's own ~/S/emacs-fun.outline  is (s)he has one : ---
-(let ((outline-file (concat (getenv "HOME") "/S/function-outline.S")))
-  (if (file-exists-p outline-file)
-      (setq ess-function-outline-file outline-file)))
-
 (defun ess-insert-function-outline ()
   "Insert an S function definition `outline' at point.
-Uses the file given by the variable ess-function-outline-file."
+Uses the file given by the variable `ess-function-outline-file'."
   (interactive)
   (let ((oldpos (point)))
     (save-excursion
@@ -386,9 +386,8 @@ Uses the file given by the variable ess-function-outline-file."
 	  (ess-rep-regexp     rgxp to nil nil verbose)
 	(query-replace-regexp rgxp to nil)))))
 
-
 (defun ess-dump-to-src (&optional dont-query verbose)
-  "Make the changes in an S - dump() file to improve human readability"
+  "Make the changes in an S - dump() file to improve human readability."
   (interactive "P")
   (save-excursion
     (if (not (equal major-mode 'ess-mode))
@@ -424,20 +423,126 @@ Uses the file given by the variable ess-function-outline-file."
 			'fixedcase 'literal verbose)
 	(setq num (1+ num))))))
 
+;;; All of the above three :
 (defun ess-MM-fix-src (&optional dont-query verbose)
   "Clean up ess-source code which has been produced by  dump(..).
- Produces more readable code, and one that is well formatted in emacs
- ess-mode. Martin Maechler, ETH Zurich."
+ Produces more readable code, and one that is well formatted in emacs ess-mode."
   (interactive "P")
-  ;; the 3 following functions each do a save-excursion:
+  ;; each of the following does a save-excursion:
   (ess-dump-to-src dont-query)
   (ess-fix-comments dont-query)
   (ess-num-var-round dont-query verbose))
 
+(defun ess-fix-miscellaneous (&optional from verbose)
+  "Fix Miscellaneous S/R `ill-formation's from current \\[point].
+ Particularly use \"<-\"and put spaces around operators."
+  (interactive "d\nP"); Defaults: point and prefix (C-u)
+  (save-excursion
+
+    (if (string= ess-dialect "R")
+	(progn
+	 (require 'essd-r)
+	 (R-fix-T-F from (not verbose))))
+
+    (goto-char from) (ess-rep-regexp " *_ *" " <- " nil 'literal verbose)
+
+    ;; ensure space around  "<-"  ---- but only replace if necessary:
+    (goto-char from)
+    (ess-rep-regexp "\\([^< \t\n]\\)\\(<<?-\\)" "\\1 \\2" nil nil verbose)
+    (goto-char from)(ess-rep-regexp "<-\\([^ \t\n]\\)" "<- \\1" nil nil verbose)
+    ;; ensure space around  "<" (not in "<-","<=","<<-")  and ">" (not ">=") :
+    (goto-char from);; --> " <", care with "->":
+    (ess-rep-regexp "\\([^-< \t\n]\\)\\([<>]\\)" "\\1 \\2" nil nil verbose)
+    ;; ">" -> "> " , for "<", don't split "<-" nor "<<-":
+    (goto-char from)
+    (ess-rep-regexp "\\(>=?\\)\\([^= \t\n]\\)" "\\1 \\2" nil nil verbose)
+    (goto-char from)
+    (ess-rep-regexp "\\(<=?\\)\\([^-<= \t\n]\\)" "\\1 \\2" nil nil t)
+
+    ;; -- ensure space around "=", "==", "!=" :
+    (goto-char from) ;; --> " ="
+    (ess-rep-regexp "\\([^=!<> ]\\)\\([=!]?\\)=" "\\1 \\2=" nil nil verbose)
+    (goto-char from) (ess-rep-regexp "=\\([^= ]\\)" "= \\1" nil nil verbose)
+
+    (goto-char from) ;; add a space between "{" and surrounding ..char:
+    (ess-rep-regexp "{\\([.A-Za-z()]\\)" "{ \\1" 'fix nil verbose)
+    (ess-rep-regexp "\\([()]\\){" "\\1 {" 'fix nil verbose)
+    (goto-char from) ;; add a space between "}" and a preceding wordchar:
+    (ess-rep-regexp "\\([A-Za-z0-9()]\\)}" "\\1 }" 'fix nil verbose)
+    (ess-space-around "else" from verbose)
+
+    ;; add a newline and indent before a "}"
+    ;; --- IFF there's NO "{" or "#" AND some NON-white text on the same line:
+    ;;D (if verbose (message "\t R-fix-misc..: Hard.. '}'"))
+    (goto-char from)
+    (ess-rep-regexp "^\\([^#{\n]*[^#{ \t\n]+[ \t]*\\)}[ \t]*$"
+		     "\\1\n}" 'fix nil verbose)
+    ))
+
+(defun ess-smart-underscore ()
+  "Electrical \"_\" key: insert `ess-S-assign' after removing spaces, unless
+ in string/comment.  `ess-S-assign', typically \" <- \", can be customized."
+  (interactive)
+  ;;(insert (if (inside-string/comment-p (point)) "_" ess-S-assign))
+  (if (inside-string/comment-p (point))
+      (insert "_")
+    ;; else
+    (delete-horizontal-space)
+    (insert ess-S-assign)))
+
+(defun ess-toggle-underscore (force)
+  "Set the \"_\" (underscore) key to \\[ess-smart-underscore] or back to \"_\".
+ Toggle the current definition, unless FORCE is non-nil, where
+ \\[ess-smart-underscore] is set unconditionally.
+
+ Using \"C-q _\" will always just insert the underscore character."
+  (interactive "P")
+  (require 'ess-mode)
+  (require 'ess-inf)
+  (let ((uscore (lookup-key ess-mode-map "_")))
+    (if (and uscore
+	     ;; (stringp uscore) (string= uscore ess-S-assign)
+	     (not force))
+	(progn
+	 (define-key ess-mode-map          "_" nil); 'self-insert-command
+	 (define-key inferior-ess-mode-map "_" nil))
+      ;; else : "force" or uscore is "nil", i.e. default
+      (define-key ess-mode-map          "_" 'ess-smart-underscore)
+      (define-key inferior-ess-mode-map "_" 'ess-smart-underscore))))
+
+;; NOTA BENE: "_" is smart *by default* :
+;; -----  The user can always customize `ess-S-assign' ...
+(ess-toggle-underscore 'force-to-S-assign)
+
 (defun ess-add-MM-keys ()
+  "Define MM's user keys, currently only \\<ess-mode-map>\\[ess-insert-function-outline]."
+  (interactive)
   (require 'ess-mode)
   (define-key ess-mode-map "\C-cf" 'ess-insert-function-outline))
 
+(defun ess-dump-args-and-go (Sfunc) ; &optional buff)
+  "Dump the function name, with arguments, to a buffer for editing.
+
+Currently, this needs to:
+   1. set the buffer to the right mode, with the right settings
+   2. format the statement,
+   3. c/function/Sfunc/
+and I need to relearn emacs lisp (but I had to, anyway."
+
+  (interactive "sFunction ? ")
+  (let* ((buffname "ess-complete.R")
+	 (buf (ess-execute (format "args(%s)" Sfunc)
+					  t
+					  buffname)))
+    (pop-to-buffer buf)
+    (message "here yet?")
+    (replace-string "function" Sfunc)
+    (ess-setq-vars-local ess-customize-alist (current-buffer))
+    (setq major-mode 'ess-mode)
+    (use-local-map ess-mode-map)
+    (set-syntax-table ess-mode-syntax-table)
+    ))
+
 
 (provide 'essl-s)
 
